﻿using Microsoft.VisualBasic;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SerialAssistant
{
    public partial class Form1 : Form
    {

        private ASCII ascii;

        private Graph Displayer;

        private Matlab_Graph matlab_graph;

        private DateTime current_time = new DateTime();    //为了避免在接收处理函数中反复调用，依然声明为一个全局变量

        private StringBuilder sb = new StringBuilder();     //为了避免在接收处理函数中反复调用，依然声明为一个全局变量

        private bool listen_status = false;//是否没有执行完invoke相关操作

        private bool close_status = false;//是否正在关闭串口，执行Application.DoEvents，并阻止再次invoke

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {

            this.Text = "串口调试助手";
            //获取电脑当前可用串口并添加到选项列表中
            port_select.Items.AddRange(System.IO.Ports.SerialPort.GetPortNames());

            //Console.WriteLine("123");
            //Console.ReadKey();

            //设置默认值
            port_select.Text = "COM1";
            baudrate_select.Text = "115200";
            databit_select.Text = "8";
            paritybit_select.Text = "None";
            stopbit_select.Text = "1";

            serial_switch.BackColor = Color.ForestGreen;
        }

        private void serial_switch_Click(object sender, EventArgs e)
        {

            try
            {
                //将可能产生异常的代码放置在try块中
                //根据当前串口属性来判断是否打开
                if (serialPort1.IsOpen)
                {

                    close_status = true;
                    while (listen_status) {
                        Application.DoEvents();
                    }
                    //串口已经处于打开状态
                    serialPort1.Close();    //关闭串口
                    close_status = false;
                    serial_switch.Text = "打开串口";
                    serial_switch.BackColor = Color.ForestGreen;
                    port_select.Enabled = true;
                    baudrate_select.Enabled = true;
                    databit_select.Enabled = true;
                    paritybit_select.Enabled = true;
                    stopbit_select.Enabled = true;
                   
                    textbox_send.Text = "";     //清空发送区
                }
                else
                {
                    //串口已经处于关闭状态，则设置好串口属性后打开
                    port_select.Enabled = false;
                    baudrate_select.Enabled = false;
                    databit_select.Enabled = false;
                    paritybit_select.Enabled = false;
                    stopbit_select.Enabled = false;
                    serialPort1.PortName = port_select.Text;
                    serialPort1.BaudRate = Convert.ToInt32(baudrate_select.Text);
                    serialPort1.DataBits = Convert.ToInt16(databit_select.Text);

                    if (paritybit_select.Text.Equals("None"))
                        serialPort1.Parity = System.IO.Ports.Parity.None;
                    else if (paritybit_select.Text.Equals("Odd"))
                        serialPort1.Parity = System.IO.Ports.Parity.Odd;
                    else if (paritybit_select.Text.Equals("Even"))
                        serialPort1.Parity = System.IO.Ports.Parity.Even;
                    else if (paritybit_select.Text.Equals("Mark"))
                        serialPort1.Parity = System.IO.Ports.Parity.Mark;
                    else if (paritybit_select.Text.Equals("Space"))
                        serialPort1.Parity = System.IO.Ports.Parity.Space;

                    if (stopbit_select.Text.Equals("1"))
                        serialPort1.StopBits = System.IO.Ports.StopBits.One;
                    else if (stopbit_select.Text.Equals("1.5"))
                        serialPort1.StopBits = System.IO.Ports.StopBits.OnePointFive;
                    else if (stopbit_select.Text.Equals("2"))
                        serialPort1.StopBits = System.IO.Ports.StopBits.Two;

                    serialPort1.Open();     //打开串口
                    serial_switch.Text = "关闭串口";
                    serial_switch.BackColor = Color.Firebrick;
                }
            }
            catch (Exception ex)
            {
                //捕获可能发生的异常并进行处理

                //捕获到异常，创建一个新的对象，之前的不可以再用
                serialPort1 = new System.IO.Ports.SerialPort();
                //刷新COM口选项
                port_select.Items.Clear();
                port_select.Items.AddRange(System.IO.Ports.SerialPort.GetPortNames());
                //响铃并显示异常给用户
                System.Media.SystemSounds.Beep.Play();
                serial_switch.Text = "打开串口";
                serial_switch.BackColor = Color.ForestGreen;
                MessageBox.Show(ex.Message);
                port_select.Enabled = true;
                baudrate_select.Enabled = true;
                databit_select.Enabled = true;
                paritybit_select.Enabled = true;
                stopbit_select.Enabled = true;
            }
        }

        private void send_Click(object sender, EventArgs e)
        {
     
            try
            {
                //首先判断串口是否开启
                if (serialPort1.IsOpen)
                {
                    int num = 0;   //获取本次发送字节数

                    
                    //判断发送模式
                    if (radioButton2.Checked)
                    {
                        //以HEX模式发送
                        //首先需要用正则表达式将用户输入字符中的十六进制字符匹配出来
                        string buf = textbox_send.Text;
                        string pattern = @"\s";
                        string replacement = "";
                        Regex rgx = new Regex(pattern);
                        string send_data = rgx.Replace(buf, replacement);


                        //Console.WriteLine(send_data);
                        //Console.WriteLine(send_data.Length);
                        //Console.ReadKey();

                        num = send_data.Length / 2;
                        byte[] temp = new byte[num];

                        for (int i = 0; i < num; i++)
                        {
                            temp[i] = Convert.ToByte(send_data.Substring(i * 2, 2), 16);
                           
                        }
                        // 接收区显示发送信息
                        textbox_receive_show(temp, " 发→");
                        serialPort1.Write(temp, 0, num);  // 向串口发送数据


                    }
                    else
                    {
                        //以ASCII模式发送
                        num = textbox_send.Text.Length;
                        textbox_receive_show(System.Text.Encoding.Default.GetBytes(textbox_send.Text), " 发→");
                        serialPort1.Write(textbox_send.Text);
                        

                    }
                }
            }
            catch (Exception ex)
            {
                //捕获到异常，创建一个新的对象，之前的不可以再用
                serialPort1 = new System.IO.Ports.SerialPort();
                //刷新COM口选项
                port_select.Items.Clear();
                port_select.Items.AddRange(System.IO.Ports.SerialPort.GetPortNames());
                //响铃并显示异常给用户
                System.Media.SystemSounds.Beep.Play();
                serial_switch.Text = "打开串口";
                serial_switch.BackColor = Color.ForestGreen;
                MessageBox.Show(ex.Message);
                port_select.Enabled = true;
                baudrate_select.Enabled = true;
                databit_select.Enabled = true;
                paritybit_select.Enabled = true;
                stopbit_select.Enabled = true;
            }
        }

        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {

            if (close_status) return;//如果正在关闭，忽略操作，直接返回，尽快的完成串口监听线程的一次循环
            listen_status = true;//设置标记，说明我已经开始处理数据，一会儿要使用系统UI的
            byte[] buf;
            do
            {
                int num = serialPort1.BytesToRead;      //获取接收缓冲区中的字节数
                buf = new byte[num];    //声明一个大小为num的字节数据用于存放读出的byte型数据

                serialPort1.Read(buf, 0, num);   //读取接收缓冲区中num个字节到byte数组中
                if (num <= 0)
                    break;

                Application.DoEvents();
                

            } while (serialPort1.BytesToRead > 0);



            textbox_receive_show(buf, " 收←");
     

        }

        private void textbox_receive_show(byte[] received_buf,String mark) {


            if (HEX.Checked)
            {
                //选中HEX模式显示
                foreach (byte b in received_buf)
                {
                    sb.Append(b.ToString("X2") + ' ');    //将byte型数据转化为2位16进制文本显示,用空格隔开
                }
            }
            else
            {
                //选中ASCII模式显示
                sb.Append(Encoding.ASCII.GetString(received_buf));  //将整个数组解码为ASCII数组
            }

            try
            {
                //因为要访问UI资源，所以需要使用invoke方式同步ui
                this.Invoke((EventHandler)(delegate
                {
                    //textbox_receive.AppendText(serialPort1.ReadExisting());

                    if (mark.Equals(" 收←") && this.Displayer != null && !this.Displayer.IsDisposed)
                    {
                        this.Displayer.AddData(received_buf);
                    }
                    if (mark.Equals(" 收←") && this.matlab_graph != null && !this.matlab_graph.IsDisposed)
                    {
                        this.matlab_graph.showGraph(received_buf);

                    }


                    if (time_check.Checked)
                    {
                        //显示时间
                        current_time = System.DateTime.Now;     //获取当前时间
                        textbox_receive.AppendText(current_time.ToString("yyyy-MM-dd HH:mm:ss") + mark + sb.ToString() + '\n');

                    }
                    else
                    {
                        //不显示时间 
                        textbox_receive.AppendText(mark + sb.ToString() + '\n');
                    }
                    sb.Clear();     //防止出错,首先清空字符串构造器

                    // 实现跟踪新接收数据
                    textbox_receive.Focus();
                }));

            }
            catch (Exception ex)
            {
                //响铃并显示异常给用户
                System.Media.SystemSounds.Beep.Play();
                MessageBox.Show(ex.Message);

            }
            finally {

                listen_status = false;//我用完了，ui可以关闭串口了。
            }



        }

        private void clean_receive_Click(object sender, EventArgs e)
        {
            textbox_receive.Text = "";
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            //定时时间到
            send_Click(send, new EventArgs());    //调用发送按钮回调函数
        }

        private void auto_send_CheckedChanged(object sender, EventArgs e)
        {


            if (auto_send.Checked)
            {
                //自动发送功能选中,开始自动发送
                time.Enabled = false;     //失能时间选择
                timer1.Interval = (int)time.Value;     //定时器赋初值
                timer1.Start();     //启动定时器

            }
            else
            {
                //自动发送功能未选中,停止自动发送
                time.Enabled = true;     //使能时间选择
                timer1.Stop();     //停止定时器


            }
        }

        private void tool1_Click(object sender, EventArgs e)
        {
            if (this.ascii == null)
            {
                this.ascii = new ASCII();
                this.ascii.Show();
                return;
            }
            if (this.ascii.IsDisposed)
            {
                this.ascii = new ASCII();
                this.ascii.Show();
                return;
            }
            this.ascii.Activate();
        }

        private void tool2_Click(object sender, EventArgs e)
        {
            if (this.Displayer == null)
            {
                this.Displayer = new Graph();
                this.Displayer.ShowUpdate += new DisplayUpdate(this.ChangeEnable);
                this.Displayer.Show();
            }
            else if (this.Displayer.IsDisposed)
            {
                this.Displayer = new Graph();
                this.Displayer.ShowUpdate += new DisplayUpdate(this.ChangeEnable);
                this.Displayer.Show();
            }
            else
            {
                this.Displayer.Activate();
            }
            
        }

        public void ChangeEnable()
        {
            
        }


        /*  ======================USB 插拔检测===========================    */
        protected override void WndProc(ref Message m)
        {

            try
            {
                //检测到USB口发生了变化,这里USB口变化时wParam的值是7，表示系统中添加或者删除了设备
                if (m.WParam.ToInt32() == 0x0007)
                {

                    port_select.Items.Clear();
                    port_select.Items.AddRange(System.IO.Ports.SerialPort.GetPortNames());

                }


                //if (m.WParam.ToInt32() == 0x8000) {         //  添加设备

                //    MessageBox.Show(" USB 插入！");
                //} else if (m.WParam.ToInt32() == 0x8004) {  //  删除设备

                //    MessageBox.Show(" USB 断开！"); 
                //}
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);　　//异常处理函数
            }
            base.WndProc(ref m);　　//这个是windos的异常处理，一定要添加，不然会运行不了
        }

        private void baudrate_select_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (baudrate_select.Text.Equals("Custom"))
            {
                string str = Interaction.InputBox("", "自定义波特率输入", "", -1, -1);
                if (!baudrate_select.Items.Contains(str)) {

                    baudrate_select.Items.Add(str);
                } 
                baudrate_select.Text = str;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (this.matlab_graph == null)
            {
                this.matlab_graph = new Matlab_Graph();
                this.matlab_graph.ShowUpdate += new DisplayUpdate(this.ChangeEnable);
                this.matlab_graph.Show();
            }
            else if (this.matlab_graph.IsDisposed)
            {
                this.matlab_graph = new Matlab_Graph();
                this.matlab_graph.ShowUpdate += new DisplayUpdate(this.ChangeEnable);
                this.matlab_graph.Show();
            }
            else
            {
                this.matlab_graph.Activate();
            }
        }


    }
}
