﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SerialAssistant
{
    public partial class Matlab_Graph : Form
    {

        //Windows API
        #region

        [DllImport("user32.dll")]

        public static extern IntPtr FindWindow(string lpClassName, string lpWindowName);

        [DllImport("user32.dll")]

        public static extern IntPtr SetParent(IntPtr hWndChild, IntPtr hWndNewParent);

        [DllImport("user32.dll", CharSet = CharSet.Auto)]

        public static extern int MoveWindow(IntPtr hWnd, int x, int y, int nWidth, int nHeight, bool BRePaint);



        const int GWL_STYLE = -16;

        const int WS_CAPTION = 0x00C00000;

        const int WS_THICKFRAME = 0x00040000;

        const int WS_SYSMENU = 0X00080000;

        [DllImport("user32")]

        private static extern int GetWindowLong(System.IntPtr hwnd, int nIndex);



        [DllImport("user32")]

        private static extern int SetWindowLong(System.IntPtr hwnd, int index, int newLong);

        [DllImport("user32")]

        private static extern int InvalidateRect(System.IntPtr hwnd, object rect, bool bErase);

        /// <summary>最大化窗口，最小化窗口，正常大小窗口

        /// nCmdShow:0隐藏,3最大化,6最小化，5正常显示

        /// </summary>

        [DllImport("user32.dll", EntryPoint = "ShowWindow")]

        public static extern int ShowWindow(IntPtr hwnd, int nCmdShow);

        #endregion

        public delegate void UpdateUI();    //委托用于更新UI        
        Thread startload;                   //线程用于matlab窗体处理              
        IntPtr figure1;                     //图像句柄        
        MLApp.MLApp matlab = null;
        public event DisplayUpdate ShowUpdate;

        public Matlab_Graph()
        {
            InitializeComponent();
        }

        private void Matlab_Graph_Load(object sender, EventArgs e)
        {
            //  把文本文件改为.m文件，只在第一次运行一次即可，或者自己手动新建.m文件放在可执行文件目录
            //FileStream fs = new FileStream("ttt.m", FileMode.OpenOrCreate, FileAccess.ReadWrite);
            //StreamWriter sw = new StreamWriter(fs); // 创建写入流
            //sw.WriteLine(Properties.Resources.ttt); // 写入Hello World
            //sw.Close(); //关闭文件
            //fs = null;

            Matlab_Init();
            Matlab_Paint(@"wave(5)");
            

            figure1 = IntPtr.Zero;
            //实例化线程，用来初次调用matlab，并把图像窗体放到winform            
            startload = new Thread(new ThreadStart(startload_run));
            startload.Start();
        }

        void Matlab_Init()
        {

            Type matlabAppType = System.Type.GetTypeFromProgID("Matlab.Application");
            matlab = System.Activator.CreateInstance(matlabAppType) as MLApp.MLApp;
        }

        void Matlab_Paint(String command)
        {

            string path_project = Directory.GetCurrentDirectory();   //工程文件的路径，如bin下面的debug
            string path_matlab = "cd('" + path_project + "')";    //自定义matlab工作路径
            matlab.Execute(path_matlab);
            matlab.Execute("clear all");
            matlab.Execute("close all");

            matlab.Execute(command);

        }

        void startload_run()
        {
            int count50ms = 0;

            //循环查找figure1窗体           
            while (figure1 == IntPtr.Zero)
            {
                //查找matlab的Figure 1窗体           
                figure1 = FindWindow("SunAwtFrame", "Figure 1");
                //延时50ms              
                Thread.Sleep(50);
                count50ms++;
                //20s超时设置              
                if (count50ms >= 400)
                {
                    return;
                }
            }
            //跨线程，用委托方式执行      
            UpdateUI update = delegate
            {

                SetParent(figure1, panel1.Handle);
                //获取窗体原来的风格             
                var style = GetWindowLong(figure1, GWL_STYLE);
                //设置新风格，去掉标题,不能通过边框改变尺寸       
                SetWindowLong(figure1, GWL_STYLE, style & ~WS_CAPTION & ~WS_THICKFRAME);
                //移动到panel里合适的位置并重绘              
                MoveWindow(figure1, 0, 0, panel1.Width + 20, panel1.Height + 40, true);
            };
            panel1.Invoke(update);
            //再移动一次，防止显示错误          
            Thread.Sleep(100);
            MoveWindow(figure1, 0, 0, panel1.Width + 20, panel1.Height + 40, true);
        }

        public void showGraph(byte[] buf) {


            string command = "wave("  + buf[0] + ")" ;

            //@"wave(170)"

            Matlab_Paint(command);
            figure1 = IntPtr.Zero;
            //实例化线程，用来初次调用matlab，并把图像窗体放到winform            
            startload_run();
        }

        // 删除.m文件 eg：DeleteFile("wave.m")
        public void DeleteFile(string path)

        {
            try
            {
                FileAttributes attr = File.GetAttributes(path);

                if (attr == FileAttributes.Directory)
                {
                    Directory.Delete(path, true);
                }
                else
                {
                    File.Delete(path);
                }
            }
            catch (FileNotFoundException ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

    }
}
